# Subtitle.pm
#
# Copyright (c) 2005 Joni Valtanen
# GNU General Public Licence v2
#

package Subtitle;

use 5.008004;
use strict;
use warnings;

require Exporter;

require Subtitle::Microdvd;
require Subtitle::Subviewer;
require Subtitle::Jacosub;

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Subtitle ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '0.4';


# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

# is numerical value
# decimal numbers are not allowed eighter
sub IsNumeric
  {
    my ($test) = @_;
    if (!($test !~ m/\D/) ) {
      return 0;
    }
    return 1;
  }

sub IsDecimal
  {
    my ($test) = @_;
    if (($test =~ m/^(\d|\.)*$/) ) {
      return 1;
    }
    return 0;
  }

sub ConvertTimeToFPS
  {
    my $line;
    my $fps;
    ($line, $fps) = @_;

    my @line_t1;
    my $time;
    my $frame;

    #time in seconds
    @line_t1 = split(/:/, $line);
    $time = $line_t1[0]*3600.0;
    $time += $line_t1[1]*60.0;
    $time += $line_t1[2];

    $frame = $time*$fps;

    return $frame;
  }

sub ConvertFPSToTime
  {
    my ($line, $fps) = @_;

    my @line_t;
    my $time;
    my $frame;
    my $secs;
    my $mins;
    my $hours;

    if(!$line) {
      $line=0;
    }
    #time in seconds
    $secs = $line / $fps * 1.0;

    $mins = int( $secs / 60.0 );
    $secs -= ( $mins * 60 );

    $hours = int( $mins / 60.0 );
    $mins -= ( $hours * 60 );

    # sprintf in Perl5 does not work like in C. Sorry
    if($secs < 10) {
      $time = sprintf("%02d:%02d:0%5.3f", $hours, $mins, $secs );
    } else {
      $time = sprintf("%02d:%02d:%5.3f", $hours, $mins, $secs );
    }
    return $time;
  }

sub GetFiletype
{
    my ( $ft ) = @_;

    my $filetype="";

    if ($ft eq "sub") {
	$filetype = "MICRODVD";
    } elsif ($ft eq "srt") {
	$filetype = "SUBVIEWER";
    } elsif ($ft eq "jss") {
	$filetype = "JACOSUB";
      }
    return ( $filetype );
}

sub GetWritableFormats
  {
    my @writable;
    my $i=0;

    if( &Subtitle::Microdvd::IsWritable() )
      {
	$writable[$i][0]="MICRODVD";
	$writable[$i++][1]="*.sub";
      }
    if( &Subtitle::Subviewer::IsWritable() )
      {
	$writable[$i][0]="SUBVIEWER";
	$writable[$i++][1]="*.srt";
      }
    if( &Subtitle::Jacosub::IsWritable() )
      {
	$writable[$i][0]="JACOSUB";
	$writable[$i++][1]="*.jss";
      }

    return ($i, @writable);

  }

sub ReadFile
  {
    my ( $filename, $fps ) = @_;

    my $lines;
    my @sub;

    if( &Subtitle::Microdvd::IsThis( $filename ) )
      {
	( $lines, @sub ) = &Subtitle::Microdvd::ReadFile( $filename, $fps );
      }
    elsif( &Subtitle::Subviewer::IsThis( $filename ) )
      {
	( $lines, @sub ) = &Subtitle::Subviewer::ReadFile( $filename, $fps );
      }
    elsif( &Subtitle::Jacosub::IsThis( $filename ) )
      {
	( $lines, @sub ) = &Subtitle::Jacosub::ReadFile( $filename, $fps );
      }

    return ( $lines, @sub );

  }

# in: $filename, $lines, $format, @subtitles
sub WriteFile
  {
    my ( $filename, $lines, $format, @subs ) = @_;

    if( &Subtitle::Microdvd::IsFormat( $format ) )
      {
	&Subtitle::Microdvd::WriteFile( $filename, $lines, @subs  );
      }
    elsif( &Subtitle::Subviewer::IsFormat( $format ) )
      {
	&Subtitle::Subviewer::WriteFile( $filename, $lines, @subs );
      }
    elsif( &Subtitle::Jacosub::IsFormat( $format ) )
      {
	&Subtitle::Jacosub::WriteFile( $filename, $lines, @subs );
      }

    return 0;

  }

# in: $filename, $lines, $format, @subtitles
sub WriteFileFromTo
  {
    my ( $filename, $from, $to, $shift, $fps, $format, @subs ) = @_;

    if( &Subtitle::Microdvd::IsFormat( $format ) )
      {
	&Subtitle::Microdvd::WriteFileFromTo( $filename, $from, $to, $shift, @subs  );
      }
    elsif( &Subtitle::Subviewer::IsFormat( $format ) )
      {
	&Subtitle::Subviewer::WriteFileFromTo( $filename, $from, $to, $shift, $fps, @subs );
      }
    elsif( &Subtitle::Jacosub::IsFormat( $format ) )
      {
	&Subtitle::Jacosub::WriteFileFromTo( $filename, $from, $to, $shift, $fps, @subs );
      }

    return 0;

  }

#in: oldfps, newfps, lines, @subtitle
sub SetNewFPS()
  {
    my ($old, $new, $lines, @subtitles) = @_;
    my @sub;
    my $d = $new/$old;
    my $i=0;
    my $j=0;
    while ( $i < $lines ) {
	if( ( int($d * $subtitles[$i]{start}) ) > 0 ) {
	    $sub[$j]{start} =  int($d*$subtitles[$i]{start});
	    $sub[$j]{end} =  int($d*$subtitles[$i]{end});
	    $sub[$j]{starttime} =  &Subtitle::ConvertFPSToTime($sub[$j]{start},$new);
	    $sub[$j]{endtime} =  &Subtitle::ConvertFPSToTime($sub[$j]{end},$new);
	    $sub[$j]{subtitle} =  $subtitles[$i]{subtitle};
	    $j++;
	  }
      $i++;
      }
    return ($j, @sub);
}

#Converts time also (stays in old fps)
#in: oldfps, newfps, lines, @subtitle
sub SetNewOldFPS()
  {
    my ($old, $new, $lines, @subtitles) = @_;
    my @sub;
    my $d = $new/$old;
    my $i=0;
    my $j=0;
    while ( $i < $lines ) {
	if( ( int($d * $subtitles[$i]{start}) ) > 0 ) {
	    $sub[$j]{start} =  int($d*$subtitles[$i]{start});
	    $sub[$j]{end} =  int($d*$subtitles[$i]{end});
	    $sub[$j]{starttime} =  &Subtitle::ConvertFPSToTime($sub[$j]{start},$old);
	    $sub[$j]{endtime} =  &Subtitle::ConvertFPSToTime($sub[$j]{end},$old);
	    $sub[$j]{subtitle} =  $subtitles[$i]{subtitle};
	    $j++;
	  }
      $i++;
      }
    return ($j, @sub);
}

# in: shifting in frames, frames from, $frames to, @subtitles in mode $subtitle[number]{start|end|starttime|endtime|subtitle}
sub Shift()
  {
    my ($shift, $from, $to, $lines, $fps, @subtitles) = @_;
    my $i=0;
    my $j=0;
    my @sub;
    if($to <= $from)
      {
	$to = $subtitles[ $lines -1]{end};
      }
    # writes whole file line by line
    while ( $i < $lines ) {
      if( ( $from <=  $subtitles[$i]{start}  ) && (  $to >=  $subtitles[$i]{end}  )  ) {
	if( ( $subtitles[$i]{start} + $shift ) > 0 ) {
	    $sub[$j]{start} =  $subtitles[$i]{start}  + $shift;
	    $sub[$j]{end} =  $subtitles[$i]{end}  + $shift;
	    $sub[$j]{subtitle} =  $subtitles[$i]{subtitle};
	    $sub[$j]{starttime} =  &Subtitle::ConvertFPSToTime( ($subtitles[$i]{start} + $shift), $fps );
	    $sub[$j]{endtime} =  &Subtitle::ConvertFPSToTime( ($subtitles[$i]{start} + $shift), $fps );
	    $j++;
	  }
      } else {
	$sub[$j]{start} =  $subtitles[$i]{start};
	$sub[$j]{end} =  $subtitles[$i]{end};
	$sub[$j]{subtitle} =  $subtitles[$i]{subtitle};
        $sub[$j]{starttime} =  $subtitles[$i]{starttime};
	$sub[$j]{endtime} =    $subtitles[$i]{endtime};
	$j++;
      }
      $i++;
    }
    return ($j, @sub);
  }

sub AddLineToEnd
{
  my ($start,$end,$subtitle,$lines,$fps,@sub) = @_;

  $sub[$lines]{start}=$start;
  $sub[$lines]{end}=$end;
  $sub[$lines]{starttime}= &Subtitle::ConvertFPSToTime($start,$fps);
  $sub[$lines]{endtime}= &Subtitle::ConvertFPSToTime($end,$fps);
  $sub[$lines]{subtitle}=$subtitle;
  $lines++;

  return ($lines,@sub)
}

sub GetKey
{
  my (@a) = @_;
  return $a[0]{start};
}

sub NormSubs
{
  my ($lines, $fps, @sub) = @_;
  my @nsub;
  my $i=0;
  my $j=0;

  for $i (0...($lines-1))
    {
      if( $sub[$i]{start} && $sub[$i]{end} )
	{
	  $nsub[$j]{start}=$sub[$i]{start};
	  $nsub[$j]{end}=$sub[$i]{end};
	  $nsub[$j]{starttime}= &Subtitle::ConvertFPSToTime($nsub[$j]{start},$fps);
	  $nsub[$j]{endtime}= &Subtitle::ConvertFPSToTime($nsub[$j]{end},$fps);
	  $nsub[$j]{subtitle}=$sub[$i]{subtitle};
	  $j++;
	}
    }
   @nsub = sort { int(GetKey($a)) <=> int(GetKey($b)) } @nsub;

  return ($j,@nsub)
}


1;
__END__

=head1 NAME

Subtitle - Perl extension for identifying, loading, saving and editing textmode subtitles

=head1 SYNOPSIS

  use Subtitle;
  # load Subviewer, MicroDVD or Jacosub subtitle
  # identifying is done automatically
   my $fps=25;
   my $filename="subtitle.sub";
   my ($lines, @sub) = &Subtitle::ReadFile($filename, $fps);
   for my $i (0...($lines-1))
   {
      print "start frame: @sub[$i]{start}\n";
      print "end frame: @sub[$i]{end}\n";
      print "start time: @sub[$i]{starttime}\n";
      print "end time: @sub[$i]{endtime}\n";
      print "subtitle: @sub[$i]{subtitle}\n";
   }

=head1 DESCRIPTION

B<Subtitle> extension allows perl developer easily read, write and edit  subtitles. Information that is needed present subtitles are starttime, stoptime and subtitle. Subtitles supported formats are B<MicroDVD>, B<SubViewer> and B<JacoSub>. Loading subtitle requires information about subtitle framerate (FPS) and subtitle name.

B<MicroDVD> is framebased format. There is used FPS to calculate start and end time of the subtitle.

B<SubViever> and B<JacoSub> are so called timebased formats. There FPS is needed to calculate frames to start and end of the single subtitle.

S<B<Subtitles internal format>>

S<start/end [ frames load( float ), save( int ) ]>

S<starttime/endtime [ hh.mm.ss.sss ( string ) ]>

S<subtitle [ line 1|line 2 ( string )]>

=head2 EXPORT

None by default.




=head1 METHODS

=over

=item IsNumeric($test)

=over

=item Tests is variable numerical. No floats are allowed.

=over

=item $test - variable to test

=back

=back

=item IsDecimal($test)

=over

=item Tests is variable float. tests only one dot.

=over

=item $test - variable to test

=back

=back

=item AddLineToEnd( $start, $end, $subtitle, $lines, $fps, @sub)

=over

=item Adds line end of the subtitles.

=over

=item $start - start frame

=item $end - end frame

=item $subtitle - subtitle text

=item $lines - howmany lines subtitles

=item $fps - FPS of the subtitlefile

=item @sub - subtitles

=back

=back

=back



=head1 SEE ALSO

Subtitle::Microdvd(3pm)
Subtitle::Subviewer(3pm)
Subtitle::Jacosub(3pm)

L<http://www.kapsi.fi/~jvaltane/subtitle/>

=head1 AUTHOR

Joni Valtanen, E<lt>jvaltane@kapsi.fiE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Joni Valtanen

This library is free software; you can redistribute it and/or modify
it under the GNU General Public Licence version 2.

Read COPYING from the package for further information.

=cut
